##############################################################################
#
# Generic Kultur configuration
#
##############################################################################
#
# Copyright 2009 Tim Brody <tdb2@ecs.soton.ac.uk>, University of Southampton
#
# This file is released under the GPLv3 license.
#

my $kultur = {};
$c->{kultur} = $kultur;

$kultur->{types} = [ qw(
	artefact
	exhibition
	composition
	performance
	image
	video
	audio
) ];

# thumbnail geometries to make available
my $geoms = {
	small => [66,50],
	medium => [200,150],
	preview => [400,300],
	lightbox => [640,480],
};
$c->{kultur}->{geoms} = $geoms;

# maximum width (in pixels) for each tab
$c->{kultur}->{max_width} = 700;

# maxiumum height (in pixels) for each tab (may overflow)
$c->{kultur}->{max_height} = 600;

# extra restrictions on how to select thumbnails for the slideshow (must be MySQL statement):
$c->{kultur}->{slideshow_restrictions} = "document.format='video' OR document.format='image' OR document.format like 'video/%' OR document.format like 'image/%'";

# generate thumbnails in order from biggest to smallest
# (possible future optimisation)
$c->{kultur}->{geom_keys} = [sort {
		$geoms->{$b}->[0] <=> $geoms->{$a}->[1]
	} keys %$geoms];

##############################################################################
#
# Kultur Tabbed Abstract Page Renderer
#
##############################################################################

######################################################################

=item $xhtmlfragment = kultur_eprint_render( $eprint, $session, $preview )

This method renders an XHTML fragment that describes the EPrint and contained documents. This generates a tabbed view.

=cut

######################################################################

# PERL complains that the methods below are redefined:
no warnings;

sub kultur_eprint_render
{
	my( $session, $eprint, $preview ) = @_;

	my $dataset = $eprint->get_dataset;

	my @docs = $eprint->get_all_documents;

	my $content = $session->make_doc_fragment;

	my $metadata_tab = {
		id => "metadata",
		title => $session->html_phrase( "page:metadata" ),
		content => $eprint->render_citation( 'summary_page_metadata' )
	};

	my @tabs = (
		# render document tab(s)
		&kultur_render_documents( $session, $dataset, $eprint, \@docs ),
		# render metadata tab(s)
		$metadata_tab
	);

	my $id_prefix = "eprint.".$eprint->get_id;

	# render the tab menu
	my @labels = map { $_->{title} } @tabs;
	my $i = 0;
	my %aliases = map { $i++ => $_->{id} } @tabs;

	$content->appendChild( $session->xhtml->tabs(
		\@labels,
		[],
		aliases => \%aliases,
		basename => $id_prefix,
		expensive => [], 
		current => 0,	#$tabs[0]->{id},
		links => {map { $_->{id} => "" } @tabs},
		base_url => $eprint->get_url,
	) );

	# panel that all the tab content sits in
	my $panel = $session->make_element( "div",
		id => "${id_prefix}_panels",
		class => "ep_tab_panel",
	);
	$content->appendChild( $panel );

	# populate each of the panels
	foreach my $tab (@tabs)
	{
		my $view_div = $session->make_element( "div",
			id => "${id_prefix}_panel_".$tab->{id},
		);
		if( $tab ne $tabs[0] )
		{
			$view_div->setAttribute( "style", "display: none" );
		}
		$view_div->appendChild( $tab->{content} );

		$panel->appendChild( $view_div );
	}

	return $content;
}

# calculate rows/columns for a given number of documents and the thumbnail
# size to use
use POSIX;
sub kultur_rows_columns
{
	my( $session, $total ) = @_;

	my $kultur = $session->get_repository->get_conf( "kultur" );
	my $geoms = $kultur->{"geoms"};
	my $geom_keys = $kultur->{"geom_keys"};
	my $max_width = $kultur->{"max_width"};
	my $max_height = $kultur->{"max_height"};

	my( $rows, $cols, $size, $text );

	# calculate the number of rows/columns for each available thumbnail size
	# and find the best fit, where we can overflow the height but not the width
	for(@$geom_keys)
	{
		$size = $_;
		my $geom = $geoms->{$_};
		$text = int($geom->[0]/6); # max chars
		$cols = int($max_width / $geom->[0]);
		$rows = int(($total / $cols)+.5);
		last if $rows * $geom->[1] <= $max_height;
	}

	return( $rows, $cols, $size, $text );
}

# render the document tab(s)
sub kultur_render_documents
{
	my( $session, $dataset, $eprint, $docs ) = @_;

	# Contact email address
	my $has_contact_email = 0;
	if(
		$session->get_repository->can_call( "email_for_doc_request" ) &&
		defined( $session->get_repository->call( "email_for_doc_request", $session, $eprint ) )
	  )
	{
		$has_contact_email = 1;
	}

	my( $doctable, $doctr, $doctd );

	my $document_groups = {};

	# assign each document to the appropriate tab
	foreach my $doc (@$docs)
	{
		my $format = $doc->get_value('format');
		if ($doc->get_value('security') ne 'public')
		{
			push @{$document_groups->{secure}}, $doc;
		}
		elsif ($format =~ /^image/)
		{
			push @{$document_groups->{image}}, $doc;
		}
		elsif ($format =~ /^(video)|(audio)/)
		{
			push @{$document_groups->{media}}, $doc;
		}
		else
		{
			push @{$document_groups->{other}}, $doc;
		}

	}

	my @tabs;

	foreach my $document_type (qw/ image media secure other /)
	{
		next unless defined $document_groups->{$document_type};

		my $total = scalar(@{$document_groups->{$document_type}});
		next if $total == 0;

		$doctable = $session->make_element( "table", class=>"ep_block" );

		my( $rows, $columns, $size, $max_text ) =
			kultur_rows_columns( $session, $total );

		my $column_count = 0;
		foreach my $doc ( @{$document_groups->{$document_type}} )
		{
			if( $column_count == 0 || $column_count >= $columns )
			{
				$doctr = $session->make_element( "tr" );
				$doctable->appendChild( $doctr );
				$column_count = 0;
			}
			++$column_count;

			$doctd = $session->make_element( "td", valign=>"top", style=>"text-align:center" );
			$doctr->appendChild( $doctd );

			my $caption = $doc->render_citation_link( "lightbox" );
			my $lightbox_caption = $session->make_doc_fragment;
			$lightbox_caption->appendChild( $caption->cloneNode( 1 ) );
			$lightbox_caption->appendChild( $eprint->render_citation_link( "lightbox" ) );
			$lightbox_caption = EPrints::XML::to_string( $lightbox_caption );
			my $link_title = EPrints::Utils::tree_to_utf8( $caption );
			EPrints::XML::dispose( $caption );

			my $ddiv = $session->make_element( "span", class => "ep_lightbox_link", title => $lightbox_caption );
			$doctd->appendChild( $ddiv );

			my $link_caption;
			if( $doc->exists_and_set( "formatdesc" ) )
			{
				$link_caption = $doc->get_value( "formatdesc" );
				$link_caption =~ s/^(.{$max_text}).*/$1 .../;
				$link_caption = $session->make_text( $link_caption );
			}
			else
			{
				#$link_caption = $doc->render_value( "format" );
				$link_caption = $session->make_text( "Untitled" );
			}
			
			if ($document_type eq 'image' || $document_type eq 'media')
			{
				my $lightbox_url =
					kultur_thumbnail_url( $doc, "video_preview" )
					|| kultur_thumbnail_url( $doc, "audio_preview" )
					|| kultur_thumbnail_url( $doc, "lightbox" );
				$lightbox_url =~ s/'/\\'/g; # must be javascript escaped


				if( !$doc->is_public )
				{
					$lightbox_url = undef;
				}

				$ddiv->appendChild( kultur_icon_link( $doc, size => $size, preview => 1, rel => "lightbox[icon_$document_type]", title => $link_title ) );

				$ddiv->appendChild( $session->make_element( 'br' ));
			
				if( $lightbox_url )
				{
					my $preview_link = $session->make_element('a', href=> $lightbox_url, rel => "lightbox[link_$document_type]", title => $link_title );
					$preview_link->appendChild( $link_caption );
					$ddiv->appendChild( $preview_link );
				}
				else
				{
					my $download_link = $session->make_element('a', href=> $doc->get_url, title => $link_title);
					if( $session->can_call( "ga_trackPageview_js" ) )
					{
						my $ga = $session->call( "ga_trackPageview_js", $session, $doc );
						$download_link->setAttribute( 'onclick', $ga ) if( EPrints::Utils::is_set( $ga ) );
					}
					$download_link->appendChild( $link_caption );
					$ddiv->appendChild( $download_link );
				}
			}
			else
			{
				$ddiv->appendChild( kultur_icon_link( $doc, size => $size, title => $link_title ) );

				$ddiv->appendChild( $session->make_element( 'br' ));
			
				if( $has_contact_email && !$doc->is_public && $eprint->get_value( "eprint_status" ) eq "archive" )
				{
# "Request a copy" button
					$doctd = $session->make_element( "td" );
					my $form = $session->render_form( "get", $session->get_repository->get_conf( "perl_url" ) . "/request_doc" );
					$form->appendChild( $session->render_hidden_field( "docid", $doc->get_id ) );
					$form->appendChild( $session->render_action_buttons( 
								"null" => $session->phrase( "request:button" )
								) );
					$doctd->appendChild( $form );
					$doctr->appendChild( $doctd );
				}
				else
				{
					my $download_link = $session->make_element('a', href=> $doc->get_url, title => $link_title);
					if( $session->can_call( "ga_trackPageview_js" ) )
					{
						my $ga = $session->call( "ga_trackPageview_js", $session, $doc );
						$download_link->setAttribute( 'onclick', $ga ) if( EPrints::Utils::is_set( $ga ) );	
					}

					$download_link->appendChild( $link_caption );
					$ddiv->appendChild( $download_link );

					my %files = $doc->files;
					if( defined $files{$doc->get_main} )
					{
						my $size = $files{$doc->get_main};
						$ddiv->appendChild( $session->make_text( " (" . EPrints::Utils::human_filesize($size) . ")" ));
					}
				}
			}

		}

		my $title = $session->html_phrase( "document_group_name_" . $document_type );

		push @tabs, {
			id => $document_type,
			title => $title,
			content => $doctable,
		};
	}

	return @tabs;
}

# render the header <link>s
sub kultur_render_links
{
	my( $session, $dataset, $eprint ) = @_;

	my $links = $session->make_doc_fragment();

	$links->appendChild( $session->plugin( "Export::Simple" )->dataobj_to_html_header( $eprint ) );
	$links->appendChild( $session->plugin( "Export::DC" )->dataobj_to_html_header( $eprint ) );

	return $links;
}

sub kultur_icon_link
{
	my( $self, %opts ) = @_;

#adam added size default to support choice of size as an option
	$opts{size} = 'small' unless defined $opts{size};

	$opts{public} = 1 unless defined $opts{public};
	if( $opts{public} && !$self->is_public )
	{
		$opts{preview} = 0;
	}

	my %aopts;
	$aopts{href} = $self->get_url;
	$aopts{title} = $opts{title};
	$aopts{target} = "_blank" if( $opts{new_window} );

	my $preview_id = "doc_preview_".$self->get_id;

	# If we have a preview available link to it instead, using the lightbox
	if ($opts{preview} and $opts{rel} and $opts{rel} =~ m/^lightbox/)
	{
		my $lightbox_url =
			kultur_thumbnail_url( $self, "video_preview" )
			|| kultur_thumbnail_url( $self, "audio_preview" )
			|| kultur_thumbnail_url( $self, "lightbox" );
		$lightbox_url =~ s/'/\\'/g; # must be javascript escaped
		if( $lightbox_url )
		{
			$aopts{href} = $lightbox_url;
			$aopts{rel} = $opts{rel};
		}
	} 

	if( !defined( $aopts{title} ) )
	{
		my $caption = $self->render_citation_link( "lightbox" );
		$aopts{title} = EPrints::Utils::tree_to_utf8( $caption );
		EPrints::XML::dispose( $caption );
	}

	my $a = $self->{session}->make_element( "a", %aopts );
	$a->appendChild( $self->{session}->make_element( 
		"img", 
		class=>"ep_doc_icon",
		alt=>"[img]",
#modified by adam to increase icon size
#		src=>$self->icon_url( public=>$opts{public} ),
		src=>$self->icon_url( public=>$opts{public},size=>$opts{size} ),
		border=>0 ));
	my $f = $self->{session}->make_doc_fragment;
	$f->appendChild( $a ) ;

	return $f;
}

sub kultur_thumbnail_url
{
	my( $doc, $resource ) = @_;

	$resource = "small" unless defined $resource;

	my $preview_doc = $doc->get_related_objects( EPrints::Utils::make_relation( "has".$resource."Version" ) )->[0];

	if (defined $preview_doc) {
		return $preview_doc->get_url;
	} else {
		return $doc->thumbnail_url($resource);
	}
}

use warnings;

